Module: Rackup::Stream::Reader

Included in:
Handler::WEBrick::Input, Rackup::Stream
Defined in:
lib/rackup/stream.rb

Overview

This provides a read-only interface for data, which is surprisingly tricky to implement correctly.

Instance Method Summary collapse

Instance Method Details

#eachObject



99
100
101
102
103
# File 'lib/rackup/stream.rb', line 99

def each
  while chunk = read_partial
    yield chunk
  end
end

#getsObject



95
96
97
# File 'lib/rackup/stream.rb', line 95

def gets
  read_partial
end

#read(length = nil, buffer = nil) ⇒ Object

read behaves like IO#read. Its signature is read([length, [buffer]]). If given, length must be a non-negative Integer (>= 0) or nil, and buffer must be a String and may not be nil. If length is given and not nil, then this method reads at most length bytes from the input stream. If length is not given or nil, then this method reads all data until EOF. When EOF is reached, this method returns nil if length is given and not nil, or “” if length is not given or is nil. If buffer is given, then the read data will be placed into buffer instead of a newly created String object.



32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
# File 'lib/rackup/stream.rb', line 32

def read(length = nil, buffer = nil)
  return '' if length == 0

  buffer ||= String.new.force_encoding(Encoding::BINARY)

  # Take any previously buffered data and replace it into the given buffer.
  if @buffer
    buffer.replace(@buffer)
    @buffer = nil
  else
    buffer.clear
  end

  if length
    while buffer.bytesize < length and chunk = read_next
      buffer << chunk
    end

    # This ensures the subsequent `slice!` works correctly.
    buffer.force_encoding(Encoding::BINARY)

    # This will be at least one copy:
    @buffer = buffer.byteslice(length, buffer.bytesize)

    # This should be zero-copy:
    buffer.slice!(length, buffer.bytesize)

    if buffer.empty?
      return nil
    else
      return buffer
    end
  else
    while chunk = read_next
      buffer << chunk
    end

    return buffer
  end
end

#read_nonblock(length, buffer = nil) ⇒ Object



105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
# File 'lib/rackup/stream.rb', line 105

def read_nonblock(length, buffer = nil)
  @buffer ||= read_next
  chunk = nil

  unless @buffer
    buffer&.clear
    return
  end

  if @buffer.bytesize > length
    chunk = @buffer.byteslice(0, length)
    @buffer = @buffer.byteslice(length, @buffer.bytesize)
  else
    chunk = @buffer
    @buffer = nil
  end

  if buffer
    buffer.replace(chunk)
  else
    buffer = chunk
  end

  return buffer
end

#read_partial(length = nil) ⇒ Object

Read at most length bytes from the stream. Will avoid reading from the underlying stream if possible.



74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
# File 'lib/rackup/stream.rb', line 74

def read_partial(length = nil)
  if @buffer
    buffer = @buffer
    @buffer = nil
  else
    buffer = read_next
  end

  if buffer and length
    if buffer.bytesize > length
      # This ensures the subsequent `slice!` works correctly.
      buffer.force_encoding(Encoding::BINARY)

      @buffer = buffer.byteslice(length, buffer.bytesize)
      buffer.slice!(length, buffer.bytesize)
    end
  end

  return buffer
end